@@ -1,5 +1,5 @@
 from pyg_base import cache, Dict, is_pd, is_arr, is_dict, dictable, cfg_read
-from pyg_sql._sql_table import sql_table, _pairs2connection, _schema, _database, get_server, pickle_loads
+from pyg_sql._sql_table import sql_table, _pairs2connection, _schema, _database, get_server
 from pyg_encoders import encode, cell_root, root_path, root_path_check, dictable_decode, WRITERS
 import pandas as pd
 import pickle
@@ -86,6 +86,9 @@ def sql_dumps(obj, path):
     # print(cursor)
     return res.path
 
+
+    
+
 def sql_loads(path):
     res = sql_binary_store(path)
     cursor = res.cursor
